#!/usr/bin/env python3

from xasyqtui.widg_addPolyOpt import Ui_Form
import PySide6.QtWidgets as QtWidgets
import PySide6.QtGui as QtGui


class Widg_addPolyOpt(QtWidgets.QWidget):
    def __init__(self, info):
        super().__init__()
        self.ui = Ui_Form()
        self.info = info
        self.ui.setupUi(self)
        self.setFixedSize(self.size())

        self.ui.chkInscribed.setChecked(self.info['inscribed'])
        self.ui.txtSides.setText(str(self.info['sides']))
        self.ui.txtSides.setValidator(QtGui.QIntValidator())

        self.ui.chkInscribed.stateChanged.connect(self.chkInscribedUpdate)
        self.ui.txtSides.textChanged.connect(self.txtSidesUpdate)

    def chkInscribedUpdate(self, checked):
        self.info['inscribed'] = checked

    def txtSidesUpdate(self, text):
        if text:
            self.info['sides'] = int(text)



#!/usr/bin/env python3
import argparse
import pathlib
import re
import sys
import subprocess
import shutil
from typing import Optional

import os

BUILD_ROOT_DIRECTORY = pathlib.Path(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(str(BUILD_ROOT_DIRECTORY.parent))

import determine_pkg_info

XASY_ICONS_MODULE_NAME = "xasyicons"

PY_UI_FILE_DIR = BUILD_ROOT_DIRECTORY / "xasyqtui"
PY_ICONS_FILE_DIR = BUILD_ROOT_DIRECTORY / XASY_ICONS_MODULE_NAME
PY_VERSION_MODULE_DIR = BUILD_ROOT_DIRECTORY / "xasyversion"
ICONS_RC_REGEX = re.compile(r"^import\s+icons_rc$")


def make_init_py_at_dir(dir_name: pathlib.Path):
    dir_name.mkdir(exist_ok=True)
    (dir_name / "__init__.py").touch(exist_ok=True)


def compile_ui_file(ui_file: pathlib.Path):
    try:
        process_result = subprocess.run(
            ["pyside6-uic", str(ui_file)],
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )
    except Exception:
        print("ERROR: PySide6 is not installed. Run 'pip install PySide6' to fix this.", file=sys.stderr)
        sys.exit(1)

    with open(
        PY_UI_FILE_DIR / ui_file.with_suffix(".py").name, "w", encoding="utf-8"
    ) as f:
        for line in process_result.stdout.splitlines():
            if ICONS_RC_REGEX.match(line.strip()) is not None:
                f.write("from xasyicons import icons_rc")
            else:
                f.write(line)
            f.write("\n")


def build_ui():
    make_init_py_at_dir(PY_UI_FILE_DIR)
    windows_dir = BUILD_ROOT_DIRECTORY / "windows"
    for ui_file in windows_dir.glob("*.ui"):
        compile_ui_file(ui_file)


def build_icons():
    PY_ICONS_FILE_DIR.mkdir(exist_ok=True)
    make_init_py_at_dir(PY_ICONS_FILE_DIR)
    try:
        subprocess.run(
            [
                "pyside6-rcc",
                str(BUILD_ROOT_DIRECTORY / "res" / "icons.qrc"),
                "-o",
                str(PY_ICONS_FILE_DIR / "icons_rc.py"),
            ],
            check=True,
        )
    except Exception:
        print("ERROR: PySide6 is not installed. Run 'pip install PySide6' to fix this.", file=sys.stderr)
        sys.exit(1)


def determine_asy_version() -> str:
    version_base = determine_pkg_info.determine_asy_pkg_info(
        BUILD_ROOT_DIRECTORY.parent / "configure.ac"
    ).get("version-base")
    if not version_base:
        return "0.0.0-SNAPSHOT"
    return version_base


def build_verison_module(version_override: Optional[str] = None):
    PY_VERSION_MODULE_DIR.mkdir(exist_ok=True)
    make_init_py_at_dir(PY_VERSION_MODULE_DIR)
    if version_override is not None:
        version = version_override
    else:
        version = determine_asy_version()
    with open(PY_VERSION_MODULE_DIR / "version.py", "w", encoding="utf-8") as f:
        f.write(f'VERSION="{version}"\n')


def clean():
    if PY_UI_FILE_DIR.exists():
        shutil.rmtree(PY_UI_FILE_DIR)

    if PY_ICONS_FILE_DIR.exists():
        shutil.rmtree(PY_ICONS_FILE_DIR)

    if PY_VERSION_MODULE_DIR.exists():
        shutil.rmtree(PY_VERSION_MODULE_DIR)


def parse_args():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help="subcommands", dest="subcommand")
    version_parser = subparsers.add_parser(
        "buildversionmodule", help="build version module"
    )
    build_parser = subparsers.add_parser("build", help="build command")
    for subparser in [build_parser, version_parser]:
        subparser.add_argument("--version-override", required=False, type=str)

    subparsers.add_parser("clean", help="clean command")
    subparsers.add_parser("buildicons", help="build icons")
    subparsers.add_parser("buildui", help="build ui files")

    return parser.parse_args()


def main():
    args = parse_args()
    if args.subcommand == "buildui":
        build_ui()
    elif args.subcommand == "buildicons":
        build_icons()
    elif args.subcommand == "buildversionmodule":
        build_verison_module(args.version_override)
    elif args.subcommand == "build":
        build_ui()
        build_icons()
        build_verison_module(args.version_override)
    elif args.subcommand == "clean":
        clean()
    else:
        raise RuntimeError("Unknown subcommand")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import argparse
from xasyversion.version import VERSION as xasyVersion
import PySide6.QtCore as QtCore
# Add arguments here.


def parseArgs(args):
    parser = argparse.ArgumentParser(args)
    parser.add_argument('-p', '-asypath', '--asypath',
                        help='Custom path to asy executable')
    parser.add_argument('-v', '-version', '--version',
                        help='Version number', action='version',
                        version='xasy v{0}'.format(xasyVersion))
    parser.add_argument('-l', '-language', '--language',
                        help='language')
    parser.add_argument('-x', '-mag', '--mag',
                        help='Initial magnification. Defaults to 1',
                        default=1, type=float)
    parser.add_argument('-render', '--render',
                        help='Number of pixels per bp in 3D rendered bitmaps',
                        default=None, type=float)
    parser.add_argument('-additional-asy-args', '--additional-asy-args',
                        help='Comma-separated values of additional arguments to pass'
                             'to Asymptote',
                        dest='additionalAsyArgs',
                        type=str, default=None)
    parser.add_argument(
        'filename',
        help='Filename to load (if omitted, initialize blank canvas)',
        nargs='?', default=None)

    return parser.parse_args()


def getArgs():
    return parseArgs(QtCore.QCoreApplication.arguments())



#!/usr/bin/env python3
###########################################################################
#
# xasyOptions provides a mechanism for storing and restoring a user's
# preferences.
#
#
# Author: Orest Shardt
# Created: June 29, 2007
#
###########################################################################

import sys
import os
import platform
import shutil
import configs
import cson

from pathlib import Path
from typing import Any

try:
    from platformdirs import user_config_dir, user_state_dir
except ModuleNotFoundError:
    def user_config_dir(appname: str, *args, **kwargs):
        path = os.environ.get("XDG_CONFIG_HOME", "")
        if not path.strip():
            path = os.path.expanduser("~/.local/share")
        return os.path.join(path, appname)

    def user_state_dir(appname: str, *args, **kwargs):
        path = os.environ.get("XDG_STATE_HOME", "")
        if not path.strip():
            path = os.path.expanduser("~/.local/state")
        return os.path.join(path, appname)


class xasyOptions:
    def __init__(self, configName: str, defaultConfigLocation: Path):
        self.configName = configName
        self._defaultOptLocation = defaultConfigLocation

        self.options: "dict[str, Any]" = {}
        self.load()

    def defaultOptions(self):
        opt: "dict[str, Any]" = cson.loads(self._defaultOptLocation.read_text())  # type: ignore
        return opt

    def overrideSettings(self):
        """Apply OS specific overrides"""
        settingsName = platform.system()

        if settingsName not in self.options:
            return

        for key in self.options[settingsName]:
            self.options[key] = self.options[settingsName][key]

    def settingsFileLocation(self):
        folders = [Path(user_config_dir("asymptote")), Path.home() / ".asy"]
        searchOrder = [".cson", ""]

        for folder in folders:
            for ext in searchOrder:
                path = folder / f"{self.configName}{ext}"
                if path.is_file():
                    return str(path)

        folders[0].mkdir(exist_ok=True, parents=True)
        return str(folders[0] / f"{self.configName}.cson")

    def __getitem__(self, key):
        return self.options[key]

    def __contains__(self, key):
        return key in self.options

    def get(self, key, default=None):
        if key not in self.options:
            return default
        return self.options[key]

    def __setitem__(self, key, value):
        self.options[key] = value

    def load(self):
        """
        Loads settings/keymaps to the `self.options` attribute.

        This method follows the order of precedence:

        1. Loads the default options.
        2. Applies the current options.
        3. Applies options from the configuration file.
        4. Applies OS-specific overrides to adjust options based on the current platform.
        """
        fileName = Path(self.settingsFileLocation())

        defaults = self.defaultOptions()

        newOptions = {}
        if fileName.exists():
            newOptions: "dict[str, Any]" = cson.loads(fileName.read_text())  # type: ignore

        # assert types match
        for source in [defaults, self.options]:
            common_keys = set(source.keys()).intersection(newOptions.keys())
            for key in common_keys:
                old_value = source[key]
                if old_value is not None:
                    assert isinstance(newOptions[key], type(old_value))

        self.options = {**defaults, **self.options, **newOptions}
        self.overrideSettings()

    def setDefaults(self):
        """Reset options/keymaps."""
        self.options = self.defaultOptions()
        if sys.platform[:3] == "win":  # for windows, wince, win32, etc
            # setAsyPathFromWindowsRegistry()
            pass

        shutil.copy2(self._defaultOptLocation, self.settingsFileLocation())


# TODO: Figure out how to merge this back.
"""
def setAsyPathFromWindowsRegistry():
    if os.name == 'nt':
        import _winreg as registry
        # test both registry locations
        try:
            key = registry.OpenKey(registry.HKEY_LOCAL_MACHINE,
                                   "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Asymptote")
            options['asyPath'] = registry.QueryValueEx(key, "Path")[0] + "\\asy.exe"
            registry.CloseKey(key)
        except:
            key = registry.OpenKey(registry.HKEY_LOCAL_MACHINE,
                                   "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Asymptote")
            options['asyPath'] = registry.QueryValueEx(key, "InstallLocation")[0] + "\\asy.exe"
            registry.CloseKey(key)
"""


class xasyOpenRecent:
    def settingsFileLocation(self, ensure_exists: bool = True):
        folders = [Path(user_state_dir("asymptote")), Path.home() / ".asy"]
        file = "xasyrecents.txt"

        for folder in folders:
            path = folder / file
            if path.exists():
                return path

        path = folders[0] / file
        if ensure_exists:
            path.parent.mkdir(exist_ok=True, parents=True)
            path.touch()
        return path

    def insert(self, path: str):
        """
        Adds a file path to the top of the recent files list,
        or moves it to the top if it already exists in the list.
        """
        path = path.strip()
        fileName = self.settingsFileLocation()

        lines = [line.strip() for line in fileName.read_text().splitlines()]

        paths = [p for p in lines if p != path]
        paths.insert(0, path)

        fileName.write_text("\n".join(paths) + "\n")

    @property
    def pathList(self):
        """Retrieves the current list of recently opened file paths.

        Loads the file paths from the storage file and ensures
        that any paths that no longer exist (i.e., are missing or deleted)
        are removed from the list.
        """

        fileName = self.settingsFileLocation()
        lines = [line.strip() for line in fileName.read_text().splitlines()]

        existing = [path for path in lines if Path(path).resolve().is_file()]
        if existing != lines:
            fileName.write_text("\n".join(existing) + "\n")

        return existing

    def clear(self):
        """Remove all items from the storage file."""
        self.settingsFileLocation().write_text("")


class BasicConfigs:
    _configPath = Path(list(configs.__path__)[0])
    defaultOpt = xasyOptions("xasyconfig", _configPath / "xasyconfig.cson")
    keymaps = xasyOptions("xasykeymap", _configPath / "xasykeymap.cson")
    openRecent = xasyOpenRecent()
